{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}root-type-name-modal{% endblock %}
{% block modal_title %}Root Type Name{% endblock %}

{% block modal_body %}
<label for="newRootName">Please provide a name for the root type:</label>
<input type="text" class="form-control" id="newRootName" placeholder="Root"/>
<div id="newRootNameError" class="text-danger">
</div>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-default pull-left" href="{% url 'core_composer_index' %}"><i class="fa fa-times"></i> Cancel</a>
<button id="rename-root-type" class="btn btn-primary" href="#"><i class="fa fa-play"></i> Start</button>
{% endblock %}
