{% extends 'core_main_app/admin/templates/versions.html' %}

{% block list_includes %}
{% include 'core_composer_app/admin/types/versions/available.html' %}
<!--FIXME: remove the control when versioning is available -->
{% if not data.version_manager.user %}
    {% include 'core_composer_app/admin/types/versions/disabled.html' %}
{% endif %}
{% endblock %}

