{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}save-success-modal{% endblock %}
{% block modal_title %}XML Schema Saved{% endblock %}

{% block modal_body %}
Saved to repository successfully.
{% endblock %}
{% block modal_footer %}
<a class="btn btn-primary" data-dismiss="modal" href="#">
    <i class="fa fa-check"></i> OK
</a>
{% endblock %}

